Documentation
¶
Overview ¶
Package forward contains all the necessary utilities for DNS request forwarding.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Handler ¶
type Handler struct {
// contains filtered or unexported fields
}
Handler is the dnscrypt.Handler implementation that forwards DNS requests to the configured upstream.
func NewHandler ¶
func NewHandler(c *HandlerConfig) (h *Handler)
NewHandler returns properly initialized *Handler. c must be non-nil and valid.
type HandlerConfig ¶
type HandlerConfig struct {
// Client is used to execute DNS requests. It must not be nil.
Client *dns.Client
// Address is the upstream address to which requests will be forwarded. It
// must not be empty.
Address netip.AddrPort
}
HandlerConfig is the configuration structure for *Handler.
Click to show internal directories.
Click to hide internal directories.